<?php
///////////////////////////////////////////////////////////////////////////

class DuneStoreModel
{
    const CATALOG_URL =
        'http://online.dune-hd.com/dune_store/v1/catalog.xml';
    const ICON_URL_PREFIX =
        'http://online.dune-hd.com/dune_store/v1/';
    const DOWNLOAD_URL_PREFIX =
        'http://files3.dune-hd.com/dune_store/v1/';
    const INSTALLED_PLUGINS_FILE =
        '/tmp/run/installed_plugins.xml';

    const MENU_CONFIG_FILE = '/firmware/config/menu_config.xml';

    const GUI_SKIN_DIR_FILE = '/tmp/dune_skin_dir.txt';

    const EXTRA_SOURCES_FIRMWARE_CONFIG_FILE =
        '/firmware/config/dsconfig.xml';
    const EXTRA_SOURCES_CONFIG_FILE =
        '/config/dsconfig.xml';

    const STANDARD_PLUGIN_SIGNER_ID = 'com.dune-hd';

    private $sources;

    private $lang;
    private $country;
    private $widget_country;
    private $product;
    private $platform_kind;

    private $plugin_names_set;
    private $skin_names_set;

    private $available_categories;
    private $available_plugin_by_name;
    private $available_skins;

    private $installed_plugins;

    private $skin_files;
    private $skin_icon_path_by_url;
    private $plugins_with_changed_badges;

    public function get_available_categories()
    {
        return $this->available_categories;
    }

    public function get_available_skins()
    {
        return $this->available_skins;
    }

    public function get_available_category_by_name($name)
    {
        foreach ($this->available_categories as $category)
        {
            if ($category->name == $name)
                return $category;
        }
        return null;
    }

    public function get_available_plugin_by_name($name)
    {
        return $this->available_plugin_by_name[$name];
    }

    public function get_available_skin_by_name($name)
    {
        foreach ($this->available_skins as $skin)
        {
            if ($skin->name == $name)
                return $skin;
        }
        return null;
    }

    public function plugin_belongs_to_category($plugin, $category)
    {
        if (!isset($category->plugins))
            return false;
        foreach ($category->plugins as $plugin_name)
        {
            if ($plugin_name == $plugin->name)
                return true;
        }
        return false;
    }

    public function skin_belongs_to_category($skin, $category)
    {
        if (!isset($category->skins))
            return false;
        foreach ($category->skins as $skin_name)
        {
            if ($skin_name == $skin->name)
                return true;
        }
        return false;
    }

    public function is_plugin_installed($plugin_name)
    {
        return isset($this->installed_plugins[$plugin_name]);
    }

    public function get_installed_plugin($plugin_name)
    {
        return $this->installed_plugins[$plugin_name];
    }

    public function is_plugin_badge_overridden($plugin_name)
    {
        return isset($this->plugins_with_changed_badges[$plugin_name]);
    }

    private function load_lang()
    {
        $lines = file("/config/settings.properties",
            FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        foreach ($lines as $line)
        {
            if (preg_match('/^interface_language = (\S+)$/', $line, $matches))
            {
                $this->lang = $matches[1];
                hd_print("Using interface language: " . $this->lang);
            }
            if (preg_match('/^widget_country = (\S.*\S)\s*$/', $line, $matches))
            {
                $this->widget_country = $matches[1];
                hd_print("Using widget_country: " . $this->widget_country);
            }
        }
        if (!isset($this->lang))
            $this->lang = "english";
        if (!isset($this->widget_country))
            $this->widget_country = "";
    }

    private function load_location_info()
    {
        $this->country = "";

        if (is_file("/tmp/location_info.properties"))
            $location_info_path = "/tmp/location_info.properties";
        else if (is_file("/config/location_info.properties"))
            $location_info_path = "/config/location_info.properties";
        else
            return;

        $lines = file($location_info_path,
            FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        foreach ($lines as $line)
        {
            if (preg_match('/^country =\s*(\S.*\S)\s*$/', $line, $matches))
            {
                $this->country = $matches[1];

                if (preg_match('/(\S[^,]*)\s*,/', $this->country, $matches))
                {
                    $this->country = $matches[1];
                }

                hd_print("Using country: " . $this->country);
                return;
            }
        }
    }

    private function load_product()
    {
        $lines = file("/tmp/run/versions.txt",
            FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        foreach ($lines as $line)
        {
            if (preg_match('/^product=\s*(\S+)\s*$/', $line, $matches))
            {
                $this->product = $matches[1];
                hd_print("Using product: " . $this->product);
            }
            else if (preg_match('/^platform_kind=\s*(\S+)\s*$/', $line, $matches))
            {
                $this->platform_kind = $matches[1];
                hd_print("Using platform_kind: " . $this->platform_kind);
            }

        }
        if (!isset($this->product))
            $this->product = "";
        if (!isset($this->platform_kind))
            $this->platform_kind = "";
    }

    private function append_standard_source(&$sources)
    {
        $sources[] = (object) array(
            'signer_id' => self::STANDARD_PLUGIN_SIGNER_ID,
            'catalog_url' => self::CATALOG_URL,
            'icon_url_prefix' => self::ICON_URL_PREFIX,
            'download_url_prefix' => self::DOWNLOAD_URL_PREFIX,
        );

        hd_print("Added standard source");
    }

    private function load_sources()
    {
        $this->sources = array();

        hd_print('Loading list of sources.');

        $dsconfig_file = null;
        if (is_file(self::EXTRA_SOURCES_CONFIG_FILE))
            $dsconfig_file = self::EXTRA_SOURCES_CONFIG_FILE;
        else if (is_file(self::EXTRA_SOURCES_FIRMWARE_CONFIG_FILE))
            $dsconfig_file = self::EXTRA_SOURCES_FIRMWARE_CONFIG_FILE;

        if ($dsconfig_file == null)
        {
            $this->append_standard_source($this->sources);
            return;
        }

        $doc = file_get_contents($dsconfig_file);
        if ($doc === false)
        {
            hd_print("Failed to load $dsconfig_file.");
            $this->append_standard_source($this->sources);
            return;
        }

        $xml = simplexml_load_string($doc);
        if ($xml === false)
        {
            hd_print("Error parsing $dsconfig_file.");
            $this->append_standard_source($this->sources);
            return;
        }

        if (!isset($xml->sources)) {
            hd_print("Warning: no 'sources' in $dsconfig_file.");
            $this->append_standard_source($this->sources);
            return;
        }

        foreach ($xml->sources->children() as $source)
        {
            $elt_name = $source->getName();
            if ($elt_name == 'standard_source') {
                $this->append_standard_source($this->sources);
                continue;
            }

            if ($elt_name != 'source') {
                hd_print("Warning: wrong element '$elt_name' inside 'sources'");
                continue;
            }

            $catalog_url = strval($source->catalog_url);
            if (!preg_match('/\S/', $catalog_url))
            {
                hd_print("Invalid source: ".
                    "required 'catalog_url' is missing.");
                continue;
            }

            if (!preg_match('/^(.*\/)[^\/]+$/', $catalog_url, $matches))
            {
                hd_print("Invalid source: ".
                    "catalog_url is invalid (1).");
                continue;
            }
            $default_prefix = $matches[1];

            hd_print("Added custom source: $catalog_url");

            $signer_id = strval($source->signer_id);
            if (!preg_match('/\S/', $signer_id))
            {
                $signer_id = null;
                hd_print("Using no signer_id for source");
            }
            else
                hd_print("Using signer_id '$signer_id' for source");

            $icon_url_prefix = strval($source->icon_url_prefix);
            if (!preg_match('/\S/', $icon_url_prefix))
                $icon_url_prefix = $default_prefix;
            hd_print("Using $icon_url_prefix as icon URL prefix");

            $download_url_prefix = strval($source->download_url_prefix);
            if (!preg_match('/\S/', $download_url_prefix))
                $download_url_prefix = $default_prefix;
            hd_print("Using $download_url_prefix as download URL prefix");

            $this->sources[] = (object) array(
                'signer_id' => $signer_id,
                'catalog_url' => $catalog_url,
                'icon_url_prefix' => $icon_url_prefix,
                'download_url_prefix' => $download_url_prefix,
            );
        }
    }

    private function to_margin($obj, $field)
    {
        if (!isset($obj->$field))
            return 0;
        return intval($obj->$field);
    }

    private function parse_caption($parent_elt, $child_name)
    {
        $default = '';
        foreach ($parent_elt->{$child_name} as $child_elt)
        {
            if (isset($child_elt['lang']))
            {
                if ($child_elt['lang'] == $this->lang)
                    return trim(strval($child_elt));
            }
            else if (isset($child_elt->default))
            {
                $default = trim(strval($child_elt->default));
            }
            else
                $default = trim(strval($child_elt));
        }
        return $default;
    }

    private function add_prefix_if_needed($value, $prefix)
    {
        $str = strval($value);

        if (preg_match('/^[A-Za-z_-]+:\/\//', $str))
            return $str;

        return $prefix . $str;
    }

    private function add_icon_prefix_if_needed($value, $prefix)
    {
        $str = strval($value);

        if (isset($this->skin_icon_path_by_url[$str]))
        {
            $path = $this->skin_icon_path_by_url[$str];
            if (isset($this->skin_files[$path]))
                return "gui_skin://plugins_special/dunestore/$path";
        }

        if (preg_match('/^[A-Za-z_-]+:\/\//', $str))
            return $str;

        if (isset($this->skin_files[$str]))
            return "gui_skin://plugins_special/dunestore/$str";

        return $prefix . $str;
    }

    private function load_catalog_for_source($source)
    {
        $signer_id = $source->signer_id;
        $catalog_url = $source->catalog_url;
        $icon_url_prefix = $source->icon_url_prefix;
        $download_url_prefix = $source->download_url_prefix;

        hd_print("Loading catalog from $catalog_url...");

        try
        {
            $doc = HD::http_get_document($catalog_url);
        }
        catch (Exception $e)
        {
            hd_print("Failed to download catalog from $catalog_url");
            return;
        }

        $xml = simplexml_load_string($doc);
        if ($xml === false)
        {
            hd_print("Failed to parse catalog from $catalog_url");
            return;
        }

        if (isset($xml->categories))
        {
            foreach ($xml->categories->children() as $category)
            {
                $cat_name = strval($category->name);
                $cat_type = strval($category->type);

                $old_cat = $this->get_available_category_by_name($cat_name);
                if ($old_cat != null && $old_cat->type != $cat_type)
                {
                    hd_print("Warning: category $cat_name type mismatch: ".
                        $old_cat->type . " != $cat_type");
                    continue;
                }

                $subcategories = null;
                $plugins = null;
                $skins = null;

                if ($cat_type == 'categories')
                {
                    $subcategories = array();
                    foreach ($category->categories->children() as $cat) {
                        $subcategories[] = strval($cat);
                    }
                }
                else if ($cat_type == 'plugins')
                {
                    $plugins = array();
                    foreach ($category->plugins->children() as $plugin) {
                        $plugin_name = strval($plugin);
                        if (!isset($this->plugin_names_set[$plugin_name])) {
                            $plugins[] = $plugin_name;
                            $plugin_names_set[$plugin_name] = 1;
                        }
                    }

                    if ($old_cat != null)
                    {
                        foreach ($plugins as $plugin) {
                            $old_cat->plugins[] = $plugin;
                            hd_print("[DEBUG] added extra plugin $plugin to $cat_name");
                        }
                    }
                }
                else if ($cat_type == 'skins')
                {
                    $skins = array();
                    foreach ($category->skins->children() as $skin) {
                        $skin_name = strval($skin);
                        if (!isset($this->skin_names_set[$skin_name])) {
                            $skins[] = $skin_name;
                            $skin_names_set[$skin_name] = 1;
                        }
                    }

                    if ($old_cat != null)
                    {
                        foreach ($skins as $skin) {
                            $old_cat->skins[] = $skin;
                            hd_print("[DEBUG] added extra skin $skin to $cat_name");
                        }
                    }
                }

                if ($old_cat != null)
                    continue;

                $this->available_categories[] = (object) array(
                    'name' => $cat_name,
                    'caption' => $this->parse_caption($category, 'caption'),
                    'type' => $cat_type,
                    'icon_url' => $this->add_icon_prefix_if_needed(
                        $category->icon_path, $icon_url_prefix),
                    'badge_icon_url' => $this->add_icon_prefix_if_needed(
                        $category->badge_icon_url, $icon_url_prefix),
                    'icon_margin_left' => $this->to_margin($category, 'icon_margin_left'),
                    'icon_margin_top' => $this->to_margin($category, 'icon_margin_top'),
                    'icon_margin_right' => $this->to_margin($category, 'icon_margin_right'),
                    'icon_margin_bottom' => $this->to_margin($category, 'icon_margin_bottom'),
                    'description' => $this->parse_caption($category, 'description'),
                    'subcategories' => $subcategories,
                    'plugins' => $plugins,
                    'skins' => $skins);
            }
        }

        if (isset($xml->plugins))
        {
            foreach ($xml->plugins->children() as $plugin)
            {
                $plugin_name = strval($plugin->name);
                if (isset($this->available_plugin_by_name[$plugin_name]))
                    continue;

                $badge_icon_url =
                    isset($this->plugins_with_changed_badges[$plugin_name]) ?
                    "gui_skin://plugins/badges/$plugin_name/default.png" :
                    $this->add_icon_prefix_if_needed(
                        $plugin->badge_icon_url, $icon_url_prefix);

                $this->available_plugin_by_name[$plugin_name] = (object) array(
                    'name' => $plugin_name,
                    'version_index' => intval($plugin->version_index),
                    'version' => strval($plugin->version),
                    'caption' => $this->parse_caption($plugin, 'caption'),
                    'icon_url' => $this->add_icon_prefix_if_needed(
                        $plugin->icon_path, $icon_url_prefix),
                    'badge_icon_url' => $badge_icon_url,
                    'poster_url' => $this->add_icon_prefix_if_needed(
                        $plugin->poster_path, $icon_url_prefix),
                    'download_url' => $this->add_prefix_if_needed(
                        $plugin->archive_path, $download_url_prefix),
                    'space_needed' => intval($plugin->space_needed),
                    'short_description' =>
                        $this->parse_caption($plugin, 'short_description'),
                    'long_description' => 
                        $this->parse_caption($plugin, 'long_description'),
                    'interface_language' => strval($plugin->interface_language),
                    'language' => strval($plugin->language),
                    'country' => strval($plugin->country),
                    'strict_country' => strval($plugin->strict_country),
                    'product' => strval($plugin->product),
                    'signer_id' => $signer_id);
            }
        }

        if (isset($xml->skins))
        {
            foreach ($xml->skins->children() as $skin)
            {
                $skin_name = strval($skin->name);
                if ($this->get_available_skin_by_name($skin_name) != null)
                    continue;

                $this->available_skins[] = (object) array(
                    'name' => $skin_name,
                    'caption' => $this->parse_caption($skin, 'caption'),
                    'icon_url' => $this->add_icon_prefix_if_needed(
                        $skin->icon_path, $icon_url_prefix),
                    'download_url' => $this->add_prefix_if_needed(
                        $skin->archive_path, $download_url_prefix),
                    'space_needed' => intval($skin->space_needed),
                    'short_description' => $this->parse_caption(
                        $skin, 'short_description'));
            }
        }
    }

    private function load_catalog()
    {
        $this->available_categories = array();
        $this->available_plugin_by_name = array();
        $this->available_skins = array();

        $this->plugin_names_set = array();
        $this->skin_names_set = array();

        foreach ($this->sources as $source) {
            $this->load_catalog_for_source($source);
        }

        hd_print('Loaded: ' .
            count($this->available_categories) . ' categories, ' .
            count($this->available_plugin_by_name) . ' plugins, ' .
            count($this->available_skins) . ' skins.');
    }

    private function load_installed_plugins()
    {
        $this->installed_plugins = array();

        hd_print('Loading list of installed plugins.');

        $doc = file_get_contents(self::INSTALLED_PLUGINS_FILE);
        if ($doc === false)
        {
            $msg = "Installed plugins file not exist.";
            hd_print($msg);
            throw new DuneException($msg, 1,
                ActionFactory::show_error(false, T::t('title_internal_error')));
            return;
        }

        $xml = simplexml_load_string($doc);
        if ($xml === false)
        {
            $msg = "Cannot parse installed-plugins file.";
            hd_print($msg);
            hd_print("Document: $doc");
            throw new DuneException($msg, 2,
                ActionFactory::show_error(false, T::t('title_internal_error')));
            return;
        }

        foreach ($xml->children() as $plugin)
        {
            $name = strval($plugin->name);
            $this->installed_plugins[$name] = $plugin;
        }

        hd_print(count($this->installed_plugins) .
            ' plugins installed.');
    }

    private function load_skin_files()
    {
        $this->skin_files = array();

        $skin_dir_props = new Properties("/tmp/dune_skin_dir.txt");
        $skin_dir_props->set_default('skin_dir', '/firmware/skin');
        $skin_dir = $skin_dir_props->skin_dir;

        $base_dir = "$skin_dir/plugins_special/dunestore";
        if (!is_dir($base_dir))
            return;

        hd_print("Reading skin files from $base_dir");

        $offset = strlen($base_dir) + 1;

        try
        {
            $it = new RecursiveIteratorIterator(
                new RecursiveDirectoryIterator($base_dir),
                RecursiveIteratorIterator::CHILD_FIRST);
            foreach ($it as $file)
            {
                if (in_array($file->getBasename(), array('.', '..'))) {
                    continue;
                } elseif ($file->isDir()) {
                    continue;
                } elseif ($file->isFile() || $file->isLink()) {

                    $path = substr($file->getPathname(), $offset);
                    hd_print("Skin file: $path");
                    $this->skin_files[$path] = 1;
                }
            }
        }
        catch (Exception $e)
        {
            hd_print("Error reading skin files: " . $e->getMessage());
        }

        hd_print("" . count($this->skin_files) . " skin files detected");
    }

    private function load_skin_icon_url_map()
    {
        $this->skin_icon_path_by_url = array();

        $skin_dir_props = new Properties("/tmp/dune_skin_dir.txt");
        $skin_dir_props->set_default('skin_dir', '/firmware/skin');
        $skin_dir = $skin_dir_props->skin_dir;

        $path = "$skin_dir/plugins_special/dunestore/icon_url_map.xml";
        if (!is_file($path))
            return;

        hd_print("Reading icon URL map from $path");

        $doc = file_get_contents($path);
        if ($doc === false)
        {
            hd_print("Failed to read $path.");
            return;
        }

        $xml = simplexml_load_string($doc);
        if ($xml === false)
        {
            hd_print("Error parsing $path.");
            return;
        }

        foreach ($xml->children() as $icon)
        {
            if ($icon->getName() != "icon") {
                hd_print("Warning: wrong elemen name; 'icon' expected");
                continue;
            }
            if (!isset($icon['url'])) {
                hd_print("Warning: icon 'url' attribute is required");
                continue;
            }
            if (!isset($icon['path'])) {
                hd_print("Warning: icon 'path' attribute is required");
                continue;
            }

            $this->skin_icon_path_by_url[strval($icon['url'])] = strval($icon['path']);
        }

        hd_print("" . count($this->skin_icon_path_by_url) .
            " icon URL mappings loaded");
    }

    private function load_skin_badges()
    {
        $this->plugins_with_changed_badges = array();

        $skin_dir_props = new Properties("/tmp/dune_skin_dir.txt");
        $skin_dir_props->set_default('skin_dir', '/firmware/skin');
        $skin_dir = $skin_dir_props->skin_dir;

        $base_dir = "$skin_dir/plugins/badges";
        if (!is_dir($base_dir))
            return;

        hd_print("Reading skin badges from $base_dir");

        $files = scandir($base_dir);
        if ($files === FALSE)
            return;

        foreach ($files as $file)
        {
            if ($file == '.' || $file == '..')
                continue;

            $path = "$base_dir/$file/default.png";
            if (!is_file($path))
                continue;

            $this->plugins_with_changed_badges[$file] = 1;
        }

        hd_print("" . count($this->plugins_with_changed_badges) .
            " plugin badges overridden by skin");
    }

    private function match_to_list($value, &$pattern)
    {
        if ($pattern === 'any')
            return true;

        $value = trim($value);
        if ($value === '')
            return false;

        $items = explode(",", $pattern);
        foreach ($items as $item)
        {
            if (strcasecmp(trim($item), $value) == 0)
                return true;
        }

        return false;
    }

    public function plugin_is_visible(&$plugin)
    {
        if (preg_match('/\S/', $plugin->strict_country))
        {
            if (!$this->match_to_list(
                    $this->country, $plugin->strict_country))
            {
                hd_print("Hiding $plugin->name by strict_country");
                return false;
            }
        }

        if (preg_match('/\S/', $plugin->country))
        {
            if ($this->widget_country !== '')
                $value = $this->widget_country;
            else
                $value = $this->country;

            if (!$this->match_to_list($value, $plugin->country))
            {
                hd_print("Hiding $plugin->name by country");
                return false;
            }
        }

        if (preg_match('/\S/', $plugin->language))
        {
            if (!$this->match_to_list(
                    $this->lang, $plugin->language))
            {
                hd_print("Hiding $plugin->name by language");
                return false;
            }
        }

        if (preg_match('/\S/', $plugin->product))
        {
            if (!$this->match_to_list(
                    $this->product, $plugin->product) &&
                !$this->match_to_list(
                    $this->platform_kind, $plugin->product))
            {
                hd_print("Hiding $plugin->name by product");
                return false;
            }
        }

        return true;
    }

    public function unset_model()
    {
        $this->lang = null;
        $this->country = null;
        $this->widget_country = null;
        $this->product = null;
        $this->platform_kind = null;

        $this->available_categories = null;
        $this->available_plugin_by_name = null;
        $this->available_skins = null;
        $this->installed_plugins = null;
        $this->skin_files = null;
        $this->skin_icon_path_by_url = null;
        $this->plugins_with_changed_badges = null;
    }

    public function ensure_loaded()
    {
        if (isset($this->installed_plugins))
            return;

        $this->load_lang();
        $this->load_location_info();
        $this->load_product();
        $this->load_skin_files();
        $this->load_skin_icon_url_map();
        $this->load_skin_badges();
        $this->load_sources();
        $this->load_catalog();
        $this->load_installed_plugins();
    }
}

///////////////////////////////////////////////////////////////////////////
?>
