<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/vod/movie.php';
require_once 'lib/screen.php';
require_once 'lib/user_input_handler_registry.php';
require_once 'lib/user_input_handler.php';
require_once 'dunestore_model.php';

///////////////////////////////////////////////////////////////////////////

class DuneStorePluginScreen implements Screen, UserInputHandler
{
    const ID = 'plugin';

    public static function get_media_url_str($category_name, $name)
    {
        return MediaURL::encode(
            array
            (
                'screen_id' => self::ID,
                'category' => $category_name,
                'name'  => $name,
            ));
    }

    ///////////////////////////////////////////////////////////////////////

    private $model;

    public function __construct(DuneStoreModel $model)
    {
        $this->model = $model;

        UserInputHandlerRegistry::get_instance()->
            register_handler($this);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_id()
    { return self::ID; }

    public function get_handler_id()
    { return self::ID; }

    ///////////////////////////////////////////////////////////////////////

    public function get_folder_view(MediaURL $media_url, &$plugin_cookies)
    {
        $this->model->ensure_loaded();

        $category = $this->model->get_available_category_by_name(
            $media_url->category);
        if (!isset($category))
            return null;

        $plugin = $this->model->get_available_plugin_by_name(
            $media_url->name);
        if (!isset($plugin))
            return null;

        $installed = $this->model->is_plugin_installed($plugin->name);
        $launch_capable = false;

        $movie = new Movie($plugin->name);
        $movie->set_data(
            $plugin->caption,
            $plugin->short_description,
            $plugin->long_description,
            $plugin->poster_url,
            -1, -1, "", "", "", "", "", "", "", "", "");
        $installed_str = $installed ?
            "%ext%<key_global>yes_no_choice_yes</key_global>" :
            "%ext%<key_global>yes_no_choice_no</key_global>";

        $details = array();
        $details[T::t('label_installed')] = $installed_str;
        if ($installed)
        {
            $installed_plugin = $this->model->get_installed_plugin($plugin->name);
            if (isset($installed_plugin->version) &&
                preg_match('/\S/', $installed_plugin->version))
            {
                $details[T::t('label_installed_version')] =
                    strval($installed_plugin->version);
            }
            $launch_capable = intval($installed_plugin->launch_capable);
        }
        if (preg_match('/\S/', $plugin->version))
            $details[T::t('label_version')] = $plugin->version;

        if (preg_match('/\S/', $plugin->interface_language))
        {
            $items = explode(",", $plugin->interface_language);
            $str = "";
            for ($ndx = 0; $ndx < count($items); $ndx++)
            {
                if ($ndx == 3)
                {
                    $str .= ', ...';
                    break;
                }

                $item = trim($items[$ndx]);
                if ($str !== "")
                    $str .= ", ";
                $str .= ucfirst($item);
            }

            $details[T::t('label_languages')] = $str;
        }

        $movie->set_details($details);
        $movie->add_series_data("1", "1", "", false);

        $button_add_params = array(
            'category' => $category->name,
            'name' => $plugin->name);

        if ($installed)
        {
            $left_button_caption = T::t('button_launch');
            $left_button_action = UserInputHandlerRegistry::create_action(
                $this, 'launch', $button_add_params);

            $has_right_button = true;
            $right_button_caption = T::t('button_uninstall');
            $right_button_action = UserInputHandlerRegistry::create_action(
                $this, 'uninstall', $button_add_params);
        }
        else
        {
            $left_button_caption = T::t('button_install');
            $left_button_action = UserInputHandlerRegistry::create_action(
                $this, 'install', $button_add_params);

            $has_right_button = false;
            $right_button_caption = null;
            $right_button_action = null;
        }

        $movie_folder_view = array
        (
            PluginMovieFolderView::movie => $movie->get_movie_array(),
            PluginMovieFolderView::left_button_caption => $left_button_caption,
            PluginMovieFolderView::left_button_action => $left_button_action,
            PluginMovieFolderView::has_right_button => $has_right_button,
            PluginMovieFolderView::right_button_caption => $right_button_caption,
            PluginMovieFolderView::right_button_action => $right_button_action,
            PluginMovieFolderView::has_multiple_series => false,
            PluginMovieFolderView::series_media_url => "none",
            PluginMovieFolderView::poster_valign => VALIGN_TOP,
        );

        return array
        (
            PluginFolderView::multiple_views_supported  => false,
            PluginFolderView::archive                   => null,
            PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_MOVIE,
            PluginFolderView::data                      => $movie_folder_view,
        );
    }


    ///////////////////////////////////////////////////////////////////////

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('PluginScreen: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        if (!isset($user_input->name))
            return null;

        if ($user_input->control_id == 'refresh')
        {
            $this->model->unset_model();
            $this->model->ensure_loaded();

            return ActionFactory::invalidate_folders(
                array(
                    DuneStorePluginListScreen::get_media_url_str(
                        $user_input->category, true),
                    DuneStorePluginListScreen::get_media_url_str(
                        $user_input->category, false),
                    self::get_media_url_str(
                        $user_input->category, $user_input->name)));
        }

        $plugin = $this->model->get_available_plugin_by_name(
            $user_input->name);
        if (!isset($plugin))
            return null;

        $add_params = array(
            'category' => $user_input->category,
            'name' => $user_input->name);
        if ($user_input->control_id == 'install')
        {
            $url = 'plugin_installer://' .
                $plugin->download_url .
                ':::name=' . $plugin->name .
                '&space_needed=' . $plugin->space_needed .
                '&need_confirm_replace=0';
            if ($plugin->signer_id != null)
                $url .= '&signer_id=' . $plugin->signer_id;

            return ActionFactory::launch_media_url($url,
                UserInputHandlerRegistry::create_action(
                    $this, 'refresh', $add_params));
        }
        else if ($user_input->control_id == 'uninstall')
        {
            $url = 'plugin_uninstaller://' . $plugin->name;
            return ActionFactory::launch_media_url($url,
                UserInputHandlerRegistry::create_action(
                    $this, 'refresh', $add_params));
        }
        else if ($user_input->control_id == 'launch')
        {
            $url = 'plugin_launcher://' . $plugin->name;
            return ActionFactory::launch_media_url($url);
        }

        return null;
    }
}

///////////////////////////////////////////////////////////////////////////
?>
